@@ -1,28 +1,32 @@
 use crate::{
     client::Client,
     error::Error,
-    request::{
-        application::{InteractionError, InteractionErrorType},
-        validate_inner, Request, RequestBuilder,
-    },
+    request::{Request, RequestBuilder, TryIntoRequest},
     response::ResponseFuture,
     routing::Route,
 };
 use serde::{ser::SerializeSeq, Serialize, Serializer};
 use twilight_model::{
     application::command::permissions::CommandPermissions,
-    id::{ApplicationId, CommandId, GuildId},
+    id::{
+        marker::{ApplicationMarker, CommandMarker, GuildMarker},
+        Id,
+    },
+};
+use twilight_validate::command::{
+    guild_permissions as validate_guild_permissions, CommandValidationError, GUILD_COMMAND_LIMIT,
+    GUILD_COMMAND_PERMISSION_LIMIT,
 };
 
 #[derive(Clone, Copy, Debug)]
 struct OptionalCommandPermissions<'a>(
-    [Option<&'a CommandPermissions>; InteractionError::GUILD_COMMAND_PERMISSION_LIMIT],
+    [Option<&'a CommandPermissions>; GUILD_COMMAND_PERMISSION_LIMIT],
 );
 
 impl OptionalCommandPermissions<'_> {
     /// Create a new list of command permissions with `None` elements.
     const fn new() -> Self {
-        Self([None; InteractionError::GUILD_COMMAND_PERMISSION_LIMIT])
+        Self([None; GUILD_COMMAND_PERMISSION_LIMIT])
     }
 
     /// Determine the number of elements present.
@@ -37,7 +41,7 @@ impl OptionalCommandPermissions<'_> {
         self.0
             .iter()
             .position(Option::is_none)
-            .unwrap_or(InteractionError::GUILD_COMMAND_PERMISSION_LIMIT)
+            .unwrap_or(GUILD_COMMAND_PERMISSION_LIMIT)
     }
 }
 
@@ -64,7 +68,7 @@ impl Serialize for OptionalCommandPermissions<'_> {
 struct SortedCommand<'a> {
     #[serde(skip_serializing)]
     count: u8,
-    id: CommandId,
+    id: Id<CommandMarker>,
     permissions: OptionalCommandPermissions<'a>,
 }
 
@@ -72,10 +76,10 @@ impl SortedCommand<'_> {
     /// Create a new default sorted command with no configured permissions.
     ///
     /// The ID of the command is `u64::MAX`.
-    fn new() -> Self {
+    const fn new() -> Self {
         Self {
             count: 0,
-            id: CommandId::new(u64::MAX).expect("non zero"),
+            id: Id::new(u64::MAX),
             permissions: OptionalCommandPermissions::new(),
         }
     }
@@ -89,14 +93,14 @@ impl SortedCommand<'_> {
 /// Sorted list of commands and their permissions.
 #[derive(Debug)]
 struct SortedCommands<'a> {
-    inner: [SortedCommand<'a>; InteractionError::GUILD_COMMAND_LIMIT],
+    inner: [SortedCommand<'a>; GUILD_COMMAND_LIMIT],
 }
 
 impl<'a> SortedCommands<'a> {
     pub fn from_pairs(
-        pairs: &'a [(CommandId, CommandPermissions)],
-    ) -> Result<Self, InteractionError> {
-        let mut sorted = [SortedCommand::new(); InteractionError::GUILD_COMMAND_LIMIT];
+        pairs: &'a [(Id<CommandMarker>, CommandPermissions)],
+    ) -> Result<Self, CommandValidationError> {
+        let mut sorted = [SortedCommand::new(); GUILD_COMMAND_LIMIT];
         let mut outer_idx = 0;
 
         'outer: while outer_idx < pairs.len() {
@@ -119,12 +123,7 @@ impl<'a> SortedCommands<'a> {
                 // if we've already reached the maximum number of command
                 // permissions allowed.
                 let sorted_count = sorted[inner_idx].count();
-
-                if !validate_inner::guild_command_permissions(sorted_count + 1) {
-                    return Err(InteractionError {
-                        kind: InteractionErrorType::TooManyCommandPermissions,
-                    });
-                }
+                validate_guild_permissions(sorted_count + 1)?;
 
                 // Set the sorted command's ID if it's currently the maximum
                 // value.
@@ -144,9 +143,7 @@ impl<'a> SortedCommands<'a> {
 
             // We've run out of space in the sorted permissions, which means the
             // user provided too many commands.
-            return Err(InteractionError {
-                kind: InteractionErrorType::TooManyCommands,
-            });
+            return Err(CommandValidationError::COMMAND_COUNT_INVALID);
         }
 
         Ok(Self { inner: sorted })
@@ -166,8 +163,8 @@ impl Serialize for SortedCommands<'_> {
 #[derive(Debug)]
 #[must_use = "requests must be configured and executed"]
 pub struct SetCommandPermissions<'a> {
-    application_id: ApplicationId,
-    guild_id: GuildId,
+    application_id: Id<ApplicationMarker>,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
     permissions: SortedCommands<'a>,
 }
@@ -175,10 +172,10 @@ pub struct SetCommandPermissions<'a> {
 impl<'a> SetCommandPermissions<'a> {
     pub(crate) fn new(
         http: &'a Client,
-        application_id: ApplicationId,
-        guild_id: GuildId,
-        permissions: &'a [(CommandId, CommandPermissions)],
-    ) -> Result<Self, InteractionError> {
+        application_id: Id<ApplicationMarker>,
+        guild_id: Id<GuildMarker>,
+        permissions: &'a [(Id<CommandMarker>, CommandPermissions)],
+    ) -> Result<Self, CommandValidationError> {
         let sorted_permissions = match SortedCommands::from_pairs(permissions) {
             Ok(sorted_permissions) => sorted_permissions,
             Err(source) => return Err(source),
@@ -192,7 +189,21 @@ impl<'a> SetCommandPermissions<'a> {
         })
     }
 
-    fn request(&self) -> Result<Request, Error> {
+    /// Execute the request, returning a future resolving to a [`Response`].
+    ///
+    /// [`Response`]: crate::response::Response
+    pub fn exec(self) -> ResponseFuture<CommandPermissions> {
+        let http = self.http;
+
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl TryIntoRequest for SetCommandPermissions<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
         Request::builder(&Route::SetCommandPermissions {
             application_id: self.application_id.get(),
             guild_id: self.guild_id.get(),
@@ -200,51 +211,41 @@ impl<'a> SetCommandPermissions<'a> {
         .json(&self.permissions)
         .map(RequestBuilder::build)
     }
-
-    /// Execute the request, returning a future resolving to a [`Response`].
-    ///
-    /// [`Response`]: crate::response::Response
-    pub fn exec(self) -> ResponseFuture<CommandPermissions> {
-        match self.request() {
-            Ok(request) => self.http.request(request),
-            Err(source) => ResponseFuture::error(source),
-        }
-    }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::{
-        super::super::{InteractionError, InteractionErrorType},
-        SetCommandPermissions,
-    };
-    use crate::Client;
+    use super::SetCommandPermissions;
+    use crate::{request::TryIntoRequest, Client};
     use serde::Deserialize;
     use std::{error::Error, iter};
     use twilight_model::{
         application::command::permissions::{CommandPermissions, CommandPermissionsType},
-        id::{ApplicationId, CommandId, GuildId, RoleId},
+        id::{
+            marker::{ApplicationMarker, CommandMarker, GuildMarker},
+            Id,
+        },
+    };
+    use twilight_validate::command::{
+        CommandValidationErrorType, GUILD_COMMAND_LIMIT, GUILD_COMMAND_PERMISSION_LIMIT,
     };
 
-    fn application_id() -> ApplicationId {
-        ApplicationId::new(1).expect("non zero")
-    }
-
-    fn guild_id() -> GuildId {
-        GuildId::new(2).expect("non zero")
-    }
+    const GUILD_ID: Id<GuildMarker> = Id::new(2);
+    const APPLICATION_ID: Id<ApplicationMarker> = Id::new(1);
 
     #[derive(Debug, Deserialize, Eq, PartialEq)]
     struct GuildCommandPermissionDeserializable {
-        id: CommandId,
+        id: Id<CommandMarker>,
         permissions: Vec<CommandPermissions>,
     }
 
-    fn command_permissions(id: CommandId) -> impl Iterator<Item = (CommandId, CommandPermissions)> {
+    fn command_permissions(
+        id: Id<CommandMarker>,
+    ) -> impl Iterator<Item = (Id<CommandMarker>, CommandPermissions)> {
         iter::repeat((
             id,
             CommandPermissions {
-                id: CommandPermissionsType::Role(RoleId::new(4).expect("non zero")),
+                id: CommandPermissionsType::Role(Id::new(4)),
                 permission: true,
             },
         ))
@@ -256,53 +257,53 @@ mod tests {
         let http = Client::new("token".to_owned());
         let command_permissions = &[
             (
-                CommandId::new(1).expect("non zero"),
+                Id::new(1),
                 CommandPermissions {
-                    id: CommandPermissionsType::Role(RoleId::new(3).expect("non zero")),
+                    id: CommandPermissionsType::Role(Id::new(3)),
                     permission: true,
                 },
             ),
             (
-                CommandId::new(1).expect("non zero"),
+                Id::new(1),
                 CommandPermissions {
-                    id: CommandPermissionsType::Role(RoleId::new(4).expect("non zero")),
+                    id: CommandPermissionsType::Role(Id::new(4)),
                     permission: true,
                 },
             ),
             (
-                CommandId::new(2).expect("non zero"),
+                Id::new(2),
                 CommandPermissions {
-                    id: CommandPermissionsType::Role(RoleId::new(5).expect("non zero")),
+                    id: CommandPermissionsType::Role(Id::new(5)),
                     permission: true,
                 },
             ),
         ];
 
         let builder =
-            SetCommandPermissions::new(&http, application_id(), guild_id(), command_permissions)?;
+            SetCommandPermissions::new(&http, APPLICATION_ID, GUILD_ID, command_permissions)?;
 
-        let request = builder.request()?;
+        let request = builder.try_into_request()?;
         let body = request.body().expect("body must be present");
         let actual = serde_json::from_slice::<Vec<GuildCommandPermissionDeserializable>>(body)?;
 
         let expected = &[
             GuildCommandPermissionDeserializable {
-                id: CommandId::new(1).expect("non zero"),
+                id: Id::new(1),
                 permissions: Vec::from([
                     CommandPermissions {
-                        id: CommandPermissionsType::Role(RoleId::new(3).expect("non zero")),
+                        id: CommandPermissionsType::Role(Id::new(3)),
                         permission: true,
                     },
                     CommandPermissions {
-                        id: CommandPermissionsType::Role(RoleId::new(4).expect("non zero")),
+                        id: CommandPermissionsType::Role(Id::new(4)),
                         permission: true,
                     },
                 ]),
             },
             GuildCommandPermissionDeserializable {
-                id: CommandId::new(2).expect("non zero"),
+                id: Id::new(2),
                 permissions: Vec::from([CommandPermissions {
-                    id: CommandPermissionsType::Role(RoleId::new(5).expect("non zero")),
+                    id: CommandPermissionsType::Role(Id::new(5)),
                     permission: true,
                 }]),
             },
@@ -316,37 +317,33 @@ mod tests {
     #[test]
     fn test_incorrect_validation() {
         let http = Client::new("token".to_owned());
-        let command_permissions = command_permissions(CommandId::new(2).expect("non zero"))
-            .take(InteractionError::GUILD_COMMAND_PERMISSION_LIMIT + 1)
+        let command_permissions = command_permissions(Id::new(2))
+            .take(GUILD_COMMAND_PERMISSION_LIMIT + 1)
             .collect::<Vec<_>>();
 
         let request =
-            SetCommandPermissions::new(&http, application_id(), guild_id(), &command_permissions);
+            SetCommandPermissions::new(&http, APPLICATION_ID, GUILD_ID, &command_permissions);
         assert!(matches!(
             request.unwrap_err().kind(),
-            InteractionErrorType::TooManyCommandPermissions
+            CommandValidationErrorType::PermissionsCountInvalid
         ));
     }
 
     #[test]
     fn test_limits() {
-        const SIZE: usize = InteractionError::GUILD_COMMAND_LIMIT;
+        const SIZE: usize = GUILD_COMMAND_LIMIT;
 
         let http = Client::new("token".to_owned());
         let command_permissions = (1..=SIZE)
             .flat_map(|id| {
-                command_permissions(CommandId::new(id as u64).expect("non zero"))
-                    .take(InteractionError::GUILD_COMMAND_PERMISSION_LIMIT)
+                command_permissions(Id::new(id as u64)).take(GUILD_COMMAND_PERMISSION_LIMIT)
             })
             .collect::<Vec<_>>();
 
-        assert!(SetCommandPermissions::new(
-            &http,
-            application_id(),
-            guild_id(),
-            &command_permissions
-        )
-        .is_ok());
+        assert!(
+            SetCommandPermissions::new(&http, APPLICATION_ID, GUILD_ID, &command_permissions)
+                .is_ok()
+        );
     }
 
     #[test]
@@ -355,16 +352,14 @@ mod tests {
 
         let http = Client::new("token".to_owned());
         let command_permissions = (1..=SIZE)
-            .flat_map(|id| {
-                command_permissions(CommandId::new(id as u64).expect("non zero")).take(3)
-            })
+            .flat_map(|id| command_permissions(Id::new(id as u64)).take(3))
             .collect::<Vec<_>>();
 
         let request =
-            SetCommandPermissions::new(&http, application_id(), guild_id(), &command_permissions);
+            SetCommandPermissions::new(&http, APPLICATION_ID, GUILD_ID, &command_permissions);
         assert!(matches!(
             request.unwrap_err().kind(),
-            InteractionErrorType::TooManyCommands
+            CommandValidationErrorType::CountInvalid
         ));
     }
 
@@ -372,6 +367,6 @@ mod tests {
     fn test_no_permissions() {
         let http = Client::new("token".to_owned());
 
-        assert!(SetCommandPermissions::new(&http, application_id(), guild_id(), &[]).is_ok());
+        assert!(SetCommandPermissions::new(&http, APPLICATION_ID, GUILD_ID, &[]).is_ok());
     }
 }
