@@ -2,41 +2,46 @@ use crate::{
     client::Client,
     error::Error as HttpError,
     request::{
-        self, attachment::AttachmentFile, multipart::Form, PartialAttachment, Request,
-        TryIntoRequest,
+        attachment::{AttachmentManager, PartialAttachment},
+        NullableField, Request, TryIntoRequest,
     },
     response::ResponseFuture,
     routing::Route,
 };
 use serde::Serialize;
-use std::borrow::Cow;
 use twilight_model::{
     application::component::Component,
     channel::{
         embed::Embed,
-        message::{AllowedMentions, MessageReference},
+        message::{AllowedMentions, MessageFlags, MessageReference},
         Message,
     },
+    http::attachment::Attachment,
     id::{
         marker::{ChannelMarker, MessageMarker, StickerMarker},
         Id,
     },
 };
 use twilight_validate::message::{
-    components as validate_components, content as validate_content, embeds as validate_embeds,
-    stickers as validate_stickers, MessageValidationError,
+    attachment_filename as validate_attachment_filename, components as validate_components,
+    content as validate_content, embeds as validate_embeds, sticker_ids as validate_sticker_ids,
+    MessageValidationError,
 };
 
 #[derive(Serialize)]
 pub(crate) struct CreateMessageFields<'a> {
-    #[serde(skip_serializing_if = "Vec::is_empty")]
-    attachments: Vec<PartialAttachment<'a>>,
-    #[serde(skip_serializing_if = "request::slice_is_empty")]
-    components: &'a [Component],
+    #[serde(skip_serializing_if = "Option::is_none")]
+    allowed_mentions: Option<NullableField<&'a AllowedMentions>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    attachments: Option<Vec<PartialAttachment<'a>>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    components: Option<&'a [Component]>,
     #[serde(skip_serializing_if = "Option::is_none")]
     content: Option<&'a str>,
-    #[serde(skip_serializing_if = "request::slice_is_empty")]
-    embeds: &'a [Embed],
+    #[serde(skip_serializing_if = "Option::is_none")]
+    embeds: Option<&'a [Embed]>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    flags: Option<MessageFlags>,
     #[serde(skip_serializing_if = "Option::is_none")]
     message_reference: Option<MessageReference>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -44,23 +49,23 @@ pub(crate) struct CreateMessageFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     payload_json: Option<&'a [u8]>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub(crate) allowed_mentions: Option<AllowedMentions>,
-    #[serde(skip_serializing_if = "request::slice_is_empty")]
-    sticker_ids: &'a [Id<StickerMarker>],
+    sticker_ids: Option<&'a [Id<StickerMarker>]>,
     #[serde(skip_serializing_if = "Option::is_none")]
     tts: Option<bool>,
 }
 
 /// Send a message to a channel.
 ///
+/// The message must include at least one of [`attachments`], [`content`],
+/// [`embeds`], or [`sticker_ids`].
+///
 /// # Example
 ///
 /// ```no_run
+/// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// use twilight_http::Client;
 /// use twilight_model::id::Id;
 ///
-/// # #[tokio::main]
-/// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 ///
 /// let channel_id = Id::new(123);
@@ -72,45 +77,79 @@ pub(crate) struct CreateMessageFields<'a> {
 ///     .await?;
 /// # Ok(()) }
 /// ```
+///
+/// [`attachments`]: Self::attachments
+/// [`content`]: Self::content
+/// [`embeds`]: Self::embeds
+/// [`sticker_ids`]: Self::sticker_ids
 #[must_use = "requests must be configured and executed"]
 pub struct CreateMessage<'a> {
-    attachments: Cow<'a, [AttachmentFile<'a>]>,
+    attachment_manager: AttachmentManager<'a>,
     channel_id: Id<ChannelMarker>,
-    pub(crate) fields: CreateMessageFields<'a>,
+    fields: CreateMessageFields<'a>,
     http: &'a Client,
 }
 
 impl<'a> CreateMessage<'a> {
     pub(crate) const fn new(http: &'a Client, channel_id: Id<ChannelMarker>) -> Self {
         Self {
+            attachment_manager: AttachmentManager::new(),
             channel_id,
             fields: CreateMessageFields {
-                attachments: Vec::new(),
-                components: &[],
+                attachments: None,
+                components: None,
                 content: None,
-                embeds: &[],
+                embeds: None,
+                flags: None,
                 message_reference: None,
                 nonce: None,
                 payload_json: None,
                 allowed_mentions: None,
-                sticker_ids: &[],
+                sticker_ids: None,
                 tts: None,
             },
-            attachments: Cow::Borrowed(&[]),
             http,
         }
     }
 
     /// Specify the [`AllowedMentions`] for the message.
-    pub fn allowed_mentions(mut self, allowed_mentions: AllowedMentions) -> Self {
-        self.fields.allowed_mentions.replace(allowed_mentions);
+    ///
+    /// Unless otherwise called, the request will use the client's default
+    /// allowed mentions. Set to `None` to ignore this default.
+    pub const fn allowed_mentions(mut self, allowed_mentions: Option<&'a AllowedMentions>) -> Self {
+        self.fields.allowed_mentions = Some(NullableField(allowed_mentions));
 
         self
     }
 
-    /// Add multiple [`Component`]s to a message.
+    /// Attach multiple files to the message.
     ///
-    /// Calling this method multiple times will clear previous calls.
+    /// Calling this method will clear previous calls.
+    ///
+    /// # Errors
+    ///
+    /// Returns an error of type [`AttachmentFilename`] if any filename is
+    /// invalid.
+    ///
+    /// [`AttachmentFilename`]: twilight_validate::message::MessageValidationErrorType::AttachmentFilename
+    pub fn attachments(
+        mut self,
+        attachments: &'a [Attachment],
+    ) -> Result<Self, MessageValidationError> {
+        attachments
+            .iter()
+            .try_for_each(|attachment| validate_attachment_filename(&attachment.filename))?;
+
+        self.attachment_manager = self
+            .attachment_manager
+            .set_files(attachments.iter().collect());
+
+        Ok(self)
+    }
+
+    /// Set the message's list of [`Component`]s.
+    ///
+    /// Calling this method will clear previous calls.
     ///
     /// # Errors
     ///
@@ -123,12 +162,12 @@ impl<'a> CreateMessage<'a> {
     ) -> Result<Self, MessageValidationError> {
         validate_components(components)?;
 
-        self.fields.components = components;
+        self.fields.components = Some(components);
 
         Ok(self)
     }
 
-    /// Set the content of the message.
+    /// Set the message's content.
     ///
     /// The maximum length is 2000 UTF-16 characters.
     ///
@@ -146,26 +185,30 @@ impl<'a> CreateMessage<'a> {
         Ok(self)
     }
 
-    /// Attach multiple embeds to the message.
+    /// Set the message's list of embeds.
     ///
-    /// Embed total character length must not exceed 6000 characters.
-    /// Additionally, the internal fields also have character limits. See
-    /// [Discord Docs/Embed Limits].
+    /// Calling this method will clear previous calls.
+    ///
+    /// The amount of embeds must not exceed [`EMBED_COUNT_LIMIT`]. The total
+    /// character length of each embed must not exceed [`EMBED_TOTAL_LENGTH`]
+    /// characters. Additionally, the internal fields also have character
+    /// limits. See [Discord Docs/Embed Limits].
     ///
     /// # Errors
     ///
     /// Returns an error of type [`TooManyEmbeds`] if there are too many embeds.
     ///
-    /// Otherwise, refer to the errors section of [`embed`] for a list of errors
-    /// that may occur.
+    /// Otherwise, refer to the errors section of
+    /// [`twilight_validate::embed::embed`] for a list of errors that may occur.
     ///
-    /// [`embed`]: twilight_validate::embed::embed
+    /// [`EMBED_COUNT_LIMIT`]: twilight_validate::message::EMBED_COUNT_LIMIT
+    /// [`EMBED_TOTAL_LENGTH`]: twilight_validate::embed::EMBED_TOTAL_LENGTH
     /// [`TooManyEmbeds`]: twilight_validate::message::MessageValidationErrorType::TooManyEmbeds
     /// [Discord Docs/Embed Limits]: https://discord.com/developers/docs/resources/channel#embed-limits
     pub fn embeds(mut self, embeds: &'a [Embed]) -> Result<Self, MessageValidationError> {
         validate_embeds(embeds)?;
 
-        self.fields.embeds = embeds;
+        self.fields.embeds = Some(embeds);
 
         Ok(self)
     }
@@ -193,22 +236,13 @@ impl<'a> CreateMessage<'a> {
         self
     }
 
-    /// Attach multiple files to the message.
+    /// Set the message's flags.
     ///
-    /// Calling this method will clear any previous calls.
-    #[allow(clippy::missing_const_for_fn)] // False positive
-    pub fn attach(mut self, attachments: &'a [AttachmentFile<'a>]) -> Self {
-        self.attachments = Cow::Borrowed(attachments);
-
-        self
-    }
-
-    /// Attach multiple files to the message.
+    /// The only supported flag is [`SUPPRESS_EMBEDS`].
     ///
-    /// Calling this method will clear any previous calls.
-    #[deprecated(since = "0.7.2", note = "Use attachments instead")]
-    pub fn files(mut self, files: &'a [(&'a str, &'a [u8])]) -> Self {
-        self.attachments = Cow::Owned(AttachmentFile::from_pairs(files));
+    /// [`SUPPRESS_EMBEDS`]: MessageFlags::SUPPRESS_EMBEDS
+    pub const fn flags(mut self, flags: MessageFlags) -> Self {
+        self.fields.flags = Some(flags);
 
         self
     }
@@ -223,10 +257,15 @@ impl<'a> CreateMessage<'a> {
     /// JSON encoded body of any additional request fields.
     ///
     /// If this method is called, all other fields are ignored, except for
-    /// [`attach`]. See [Discord Docs/Create Message].
+    /// [`attachments`]. See [Discord Docs/Uploading Files].
     ///
-    /// [`attach`]: Self::attach
-    /// [Discord Docs/Create Message]: https://discord.com/developers/docs/resources/channel#create-message-params
+    /// # Examples
+    ///
+    /// See [`ExecuteWebhook::payload_json`] for examples.
+    ///
+    /// [Discord Docs/Uploading Files]: https://discord.com/developers/docs/reference#uploading-files
+    /// [`ExecuteWebhook::payload_json`]: crate::request::channel::webhook::ExecuteWebhook::payload_json
+    /// [`attachments`]: Self::attachments
     pub const fn payload_json(mut self, payload_json: &'a [u8]) -> Self {
         self.fields.payload_json = Some(payload_json);
 
@@ -259,13 +298,6 @@ impl<'a> CreateMessage<'a> {
         self
     }
 
-    /// Specify true if the message is TTS.
-    pub const fn tts(mut self, tts: bool) -> Self {
-        self.fields.tts = Some(tts);
-
-        self
-    }
-
     /// Set the IDs of up to 3 guild stickers.
     ///
     /// # Errors
@@ -273,17 +305,24 @@ impl<'a> CreateMessage<'a> {
     /// Returns an error of type [`StickersInvalid`] if the length is invalid.
     ///
     /// [`StickersInvalid`]: twilight_validate::message::MessageValidationErrorType::StickersInvalid
-    pub fn stickers(
+    pub fn sticker_ids(
         mut self,
-        stickers: &'a [Id<StickerMarker>],
+        sticker_ids: &'a [Id<StickerMarker>],
     ) -> Result<Self, MessageValidationError> {
-        validate_stickers(stickers)?;
+        validate_sticker_ids(sticker_ids)?;
 
-        self.fields.sticker_ids = stickers;
+        self.fields.sticker_ids = Some(sticker_ids);
 
         Ok(self)
     }
 
+    /// Specify true if the message is TTS.
+    pub const fn tts(mut self, tts: bool) -> Self {
+        self.fields.tts = Some(tts);
+
+        self
+    }
+
     /// Execute the request, returning a future resolving to a [`Response`].
     ///
     /// [`Response`]: crate::response::Response
@@ -303,33 +342,29 @@ impl TryIntoRequest for CreateMessage<'_> {
             channel_id: self.channel_id.get(),
         });
 
-        if !self.attachments.is_empty() || self.fields.payload_json.is_some() {
-            let mut form = Form::new();
-
-            if !self.attachments.is_empty() {
-                for (index, attachment) in self.attachments.iter().enumerate() {
-                    form.attach(
-                        index as u64,
-                        attachment.filename.as_bytes(),
-                        attachment.file,
-                    );
-                    self.fields.attachments.push(PartialAttachment {
-                        id: index as u64,
-                        filename: attachment.filename,
-                        description: attachment.description,
-                    })
-                }
+        // Set the default allowed mentions if required.
+        if self.fields.allowed_mentions.is_none() {
+            if let Some(allowed_mentions) = self.http.default_allowed_mentions() {
+                self.fields.allowed_mentions = Some(NullableField(Some(allowed_mentions)));
             }
+        }
 
-            if let Some(payload_json) = &self.fields.payload_json {
-                form.payload_json(payload_json);
+        // Determine whether we need to use a multipart/form-data body or a JSON
+        // body.
+        if !self.attachment_manager.is_empty() {
+            let form = if let Some(payload_json) = self.fields.payload_json {
+                self.attachment_manager.build_form(payload_json)
             } else {
-                let body = crate::json::to_vec(&self.fields).map_err(HttpError::json)?;
+                self.fields.attachments = Some(self.attachment_manager.get_partial_attachments());
 
-                form.payload_json(&body);
-            }
+                let fields = crate::json::to_vec(&self.fields).map_err(HttpError::json)?;
+
+                self.attachment_manager.build_form(fields.as_ref())
+            };
 
             request = request.form(form);
+        } else if let Some(payload_json) = self.fields.payload_json {
+            request = request.body(payload_json.to_vec())
         } else {
             request = request.json(&self.fields)?;
         }
