@@ -1,30 +1,38 @@
-use super::ExecuteWebhookAndWait;
 use crate::{
     client::Client,
     error::Error as HttpError,
-    request::{AttachmentFile, Form, PartialAttachment, Request, TryIntoRequest},
+    request::{
+        attachment::{AttachmentManager, PartialAttachment},
+        channel::webhook::ExecuteWebhookAndWait,
+        NullableField, Request, TryIntoRequest,
+    },
     response::{marker::EmptyBody, ResponseFuture},
     routing::Route,
 };
 use serde::Serialize;
-use std::borrow::Cow;
 use twilight_model::{
     application::component::Component,
-    channel::{embed::Embed, message::AllowedMentions},
+    channel::{
+        embed::Embed,
+        message::{AllowedMentions, MessageFlags},
+    },
+    http::attachment::Attachment,
     id::{
         marker::{ChannelMarker, WebhookMarker},
         Id,
     },
 };
 use twilight_validate::message::{
-    components as validate_components, content as validate_content, embeds as validate_embeds,
-    MessageValidationError,
+    attachment_filename as validate_attachment_filename, components as validate_components,
+    content as validate_content, embeds as validate_embeds, MessageValidationError,
 };
 
 #[derive(Serialize)]
 pub(crate) struct ExecuteWebhookFields<'a> {
-    #[serde(skip_serializing_if = "Vec::is_empty")]
-    attachments: Vec<PartialAttachment<'a>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    allowed_mentions: Option<NullableField<&'a AllowedMentions>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    attachments: Option<Vec<PartialAttachment<'a>>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     avatar_url: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -34,27 +42,27 @@ pub(crate) struct ExecuteWebhookFields<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     embeds: Option<&'a [Embed]>,
     #[serde(skip_serializing_if = "Option::is_none")]
+    flags: Option<MessageFlags>,
+    #[serde(skip_serializing_if = "Option::is_none")]
     payload_json: Option<&'a [u8]>,
     #[serde(skip_serializing_if = "Option::is_none")]
     tts: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
     username: Option<&'a str>,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    pub(crate) allowed_mentions: Option<AllowedMentions>,
 }
 
 /// Execute a webhook, sending a message to its channel.
 ///
-/// You can only specify one of [`content`], [`embeds`], or [`files`].
+/// The message must include at least one of [`attachments`], [`content`], or
+/// [`embeds`].
 ///
 /// # Examples
 ///
 /// ```no_run
+/// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// use twilight_http::Client;
 /// use twilight_model::id::Id;
 ///
-/// # #[tokio::main]
-/// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// let client = Client::new("my token".to_owned());
 /// let id = Id::new(432);
 ///
@@ -66,16 +74,17 @@ pub(crate) struct ExecuteWebhookFields<'a> {
 /// # Ok(()) }
 /// ```
 ///
+/// [`attachments`]: Self::attachments
 /// [`content`]: Self::content
 /// [`embeds`]: Self::embeds
-/// [`files`]: Self::files
 #[must_use = "requests must be configured and executed"]
 pub struct ExecuteWebhook<'a> {
-    attachments: Cow<'a, [AttachmentFile<'a>]>,
-    pub(crate) fields: ExecuteWebhookFields<'a>,
-    pub(super) http: &'a Client,
+    attachment_manager: AttachmentManager<'a>,
+    fields: ExecuteWebhookFields<'a>,
+    http: &'a Client,
     thread_id: Option<Id<ChannelMarker>>,
     token: &'a str,
+    wait: bool,
     webhook_id: Id<WebhookMarker>,
 }
 
@@ -86,32 +95,62 @@ impl<'a> ExecuteWebhook<'a> {
         token: &'a str,
     ) -> Self {
         Self {
+            attachment_manager: AttachmentManager::new(),
             fields: ExecuteWebhookFields {
-                attachments: Vec::new(),
+                attachments: None,
                 avatar_url: None,
                 components: None,
                 content: None,
                 embeds: None,
+                flags: None,
                 payload_json: None,
                 tts: None,
                 username: None,
                 allowed_mentions: None,
             },
-            attachments: Cow::Borrowed(&[]),
             http,
             thread_id: None,
             token,
+            wait: false,
             webhook_id,
         }
     }
 
-    /// Specify the [`AllowedMentions`] for the webhook message.
-    pub fn allowed_mentions(mut self, allowed_mentions: AllowedMentions) -> Self {
-        self.fields.allowed_mentions.replace(allowed_mentions);
+    /// Specify the [`AllowedMentions`] for the message.
+    ///
+    /// Unless otherwise called, the request will use the client's default
+    /// allowed mentions. Set to `None` to ignore this default.
+    pub const fn allowed_mentions(mut self, allowed_mentions: Option<&'a AllowedMentions>) -> Self {
+        self.fields.allowed_mentions = Some(NullableField(allowed_mentions));
 
         self
     }
 
+    /// Attach multiple files to the message.
+    ///
+    /// Calling this method will clear any previous calls.
+    ///
+    /// # Errors
+    ///
+    /// Returns an error of type [`AttachmentFilename`] if any filename is
+    /// invalid.
+    ///
+    /// [`AttachmentFilename`]: twilight_validate::message::MessageValidationErrorType::AttachmentFilename
+    pub fn attachments(
+        mut self,
+        attachments: &'a [Attachment],
+    ) -> Result<Self, MessageValidationError> {
+        attachments
+            .iter()
+            .try_for_each(|attachment| validate_attachment_filename(&attachment.filename))?;
+
+        self.attachment_manager = self
+            .attachment_manager
+            .set_files(attachments.iter().collect());
+
+        Ok(self)
+    }
+
     /// The URL of the avatar of the webhook.
     pub const fn avatar_url(mut self, avatar_url: &'a str) -> Self {
         self.fields.avatar_url = Some(avatar_url);
@@ -119,9 +158,11 @@ impl<'a> ExecuteWebhook<'a> {
         self
     }
 
-    /// Add multiple [`Component`]s to a message.
+    /// Set the message's list of [`Component`]s.
     ///
-    /// Calling this method multiple times will clear previous calls.
+    /// Calling this method will clear previous calls.
+    ///
+    /// Requires a webhook owned by the application.
     ///
     /// # Errors
     ///
@@ -139,9 +180,9 @@ impl<'a> ExecuteWebhook<'a> {
         Ok(self)
     }
 
-    /// The content of the webhook's message.
+    /// Set the message's content.
     ///
-    /// Up to 2000 UTF-16 codepoints, same as a message.
+    /// The maximum length is 2000 UTF-16 characters.
     ///
     /// # Errors
     ///
@@ -157,17 +198,26 @@ impl<'a> ExecuteWebhook<'a> {
         Ok(self)
     }
 
-    /// Set the list of embeds of the webhook's message.
+    /// Set the message's list of embeds.
+    ///
+    /// Calling this method will clear previous calls.
+    ///
+    /// The amount of embeds must not exceed [`EMBED_COUNT_LIMIT`]. The total
+    /// character length of each embed must not exceed [`EMBED_TOTAL_LENGTH`]
+    /// characters. Additionally, the internal fields also have character
+    /// limits. Refer to [Discord Docs/Embed Limits] for more information.
     ///
     /// # Errors
     ///
     /// Returns an error of type [`TooManyEmbeds`] if there are too many embeds.
     ///
-    /// Otherwise, refer to the errors section of [`embed`] for a list of errors
-    /// that may occur.
+    /// Otherwise, refer to the errors section of
+    /// [`twilight_validate::embed::embed`] for a list of errors that may occur.
     ///
+    /// [Discord Docs/Embed Limits]: https://discord.com/developers/docs/resources/channel#embed-limits
+    /// [`EMBED_COUNT_LIMIT`]: twilight_validate::message::EMBED_COUNT_LIMIT
+    /// [`EMBED_TOTAL_LENGTH`]: twilight_validate::embed::EMBED_TOTAL_LENGTH
     /// [`TooManyEmbeds`]: twilight_validate::message::MessageValidationErrorType::TooManyEmbeds
-    /// [`embed`]: twilight_validate::embed::embed
     pub fn embeds(mut self, embeds: &'a [Embed]) -> Result<Self, MessageValidationError> {
         validate_embeds(embeds)?;
 
@@ -176,22 +226,13 @@ impl<'a> ExecuteWebhook<'a> {
         Ok(self)
     }
 
-    /// Attach multiple files to the message.
+    /// Set the message's flags.
     ///
-    /// Calling this method will clear any previous calls.
-    #[allow(clippy::missing_const_for_fn)] // False positive
-    pub fn attach(mut self, attachments: &'a [AttachmentFile<'a>]) -> Self {
-        self.attachments = Cow::Borrowed(attachments);
-
-        self
-    }
-
-    /// Attach multiple files to the message.
+    /// The only supported flag is [`SUPPRESS_EMBEDS`].
     ///
-    /// Calling this method will clear any previous calls.
-    #[deprecated(since = "0.7.2", note = "Use attach instead")]
-    pub fn files(mut self, files: &'a [(&'a str, &'a [u8])]) -> Self {
-        self.attachments = Cow::Owned(AttachmentFile::from_pairs(files));
+    /// [`SUPPRESS_EMBEDS`]: MessageFlags::SUPPRESS_EMBEDS
+    pub const fn flags(mut self, flags: MessageFlags) -> Self {
+        self.fields.flags = Some(flags);
 
         self
     }
@@ -199,20 +240,18 @@ impl<'a> ExecuteWebhook<'a> {
     /// JSON encoded body of any additional request fields.
     ///
     /// If this method is called, all other fields are ignored, except for
-    /// [`attach`]. See [Discord Docs/Create Message].
-    ///
-    /// # Examples
+    /// [`attachments`]. See [Discord Docs/Uploading Files].
     ///
     /// Without [`payload_json`]:
     ///
     /// ```no_run
+    /// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// use twilight_http::Client;
     /// use twilight_embed_builder::EmbedBuilder;
-    /// # use twilight_http::Client;
     /// use twilight_model::id::Id;
     ///
-    /// # #[tokio::main]
-    /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let client = Client::new("token".to_owned());
+    /// let client = Client::new("token".to_owned());
+    ///
     /// let message = client.execute_webhook(Id::new(1), "token here")
     ///     .content("some content")?
     ///     .embeds(&[EmbedBuilder::new().title("title").build()?])?
@@ -229,12 +268,13 @@ impl<'a> ExecuteWebhook<'a> {
     /// With [`payload_json`]:
     ///
     /// ```no_run
-    /// # use twilight_http::Client;
+    /// # #[tokio::main] async fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// use twilight_http::Client;
+    /// use twilight_embed_builder::EmbedBuilder;
     /// use twilight_model::id::Id;
     ///
-    /// # #[tokio::main]
-    /// # async fn main() -> Result<(), Box<dyn std::error::Error>> {
-    /// # let client = Client::new("token".to_owned());
+    /// let client = Client::new("token".to_owned());
+    ///
     /// let message = client.execute_webhook(Id::new(1), "token here")
     ///     .content("some content")?
     ///     .payload_json(br#"{ "content": "other content", "embeds": [ { "title": "title" } ] }"#)
@@ -248,9 +288,9 @@ impl<'a> ExecuteWebhook<'a> {
     /// # Ok(()) }
     /// ```
     ///
-    /// [`attach`]: Self::attach
+    /// [Discord Docs/Uploading Files]: https://discord.com/developers/docs/reference#uploading-files
+    /// [`attachments`]: Self::attachments
     /// [`payload_json`]: Self::payload_json
-    /// [Discord Docs/Create Message]: https://discord.com/developers/docs/resources/channel#create-message-params
     pub const fn payload_json(mut self, payload_json: &'a [u8]) -> Self {
         self.fields.payload_json = Some(payload_json);
 
@@ -284,57 +324,10 @@ impl<'a> ExecuteWebhook<'a> {
     /// Using this will result in receiving the created message.
     ///
     /// [Discord Docs/Execute Webhook]: https://discord.com/developers/docs/resources/webhook#execute-webhook-querystring-params
-    #[allow(clippy::missing_const_for_fn)]
-    pub fn wait(self) -> ExecuteWebhookAndWait<'a> {
-        ExecuteWebhookAndWait::new(self)
-    }
+    pub const fn wait(mut self) -> ExecuteWebhookAndWait<'a> {
+        self.wait = true;
 
-    // `self` needs to be consumed and the client returned due to parameters
-    // being consumed in request construction.
-    pub(super) fn request(&mut self, wait: bool) -> Result<Request, HttpError> {
-        let mut request = Request::builder(&Route::ExecuteWebhook {
-            thread_id: self.thread_id.map(Id::get),
-            token: self.token,
-            wait: Some(wait),
-            webhook_id: self.webhook_id.get(),
-        });
-
-        // Webhook executions don't need the authorization token, only the
-        // webhook token.
-        request = request.use_authorization_token(false);
-
-        if !self.attachments.is_empty() || self.fields.payload_json.is_some() {
-            let mut form = Form::new();
-
-            if !self.attachments.is_empty() {
-                for (index, attachment) in self.attachments.iter().enumerate() {
-                    form.attach(
-                        index as u64,
-                        attachment.filename.as_bytes(),
-                        attachment.file,
-                    );
-                    self.fields.attachments.push(PartialAttachment {
-                        id: index as u64,
-                        filename: attachment.filename,
-                        description: attachment.description,
-                    })
-                }
-            }
-
-            if let Some(payload_json) = &self.fields.payload_json {
-                form.payload_json(payload_json);
-            } else {
-                let body = crate::json::to_vec(&self.fields).map_err(HttpError::json)?;
-
-                form.payload_json(&body);
-            }
-
-            request = request.form(form);
-        } else {
-            request = request.json(&self.fields)?;
-        }
-
-        Ok(request.build())
+        ExecuteWebhookAndWait::new(self.http, self)
     }
 
     /// Execute the request, returning a future resolving to a [`Response`].
@@ -352,6 +345,44 @@ impl<'a> ExecuteWebhook<'a> {
 
 impl TryIntoRequest for ExecuteWebhook<'_> {
     fn try_into_request(mut self) -> Result<Request, HttpError> {
-        self.request(false)
+        let mut request = Request::builder(&Route::ExecuteWebhook {
+            thread_id: self.thread_id.map(Id::get),
+            token: self.token,
+            wait: Some(self.wait),
+            webhook_id: self.webhook_id.get(),
+        });
+
+        // Webhook executions don't need the authorization token, only the
+        // webhook token.
+        request = request.use_authorization_token(false);
+
+        // Set the default allowed mentions if required.
+        if self.fields.allowed_mentions.is_none() {
+            if let Some(allowed_mentions) = self.http.default_allowed_mentions() {
+                self.fields.allowed_mentions = Some(NullableField(Some(allowed_mentions)));
+            }
+        }
+
+        // Determine whether we need to use a multipart/form-data body or a JSON
+        // body.
+        if !self.attachment_manager.is_empty() {
+            let form = if let Some(payload_json) = self.fields.payload_json {
+                self.attachment_manager.build_form(payload_json)
+            } else {
+                self.fields.attachments = Some(self.attachment_manager.get_partial_attachments());
+
+                let fields = crate::json::to_vec(&self.fields).map_err(HttpError::json)?;
+
+                self.attachment_manager.build_form(fields.as_ref())
+            };
+
+            request = request.form(form);
+        } else if let Some(payload_json) = self.fields.payload_json {
+            request = request.body(payload_json.to_vec())
+        } else {
+            request = request.json(&self.fields)?;
+        }
+
+        Ok(request.build())
     }
 }
