@@ -1,5 +1,14 @@
-use crate::{client::Client, request::Request, response::ResponseFuture, routing::Route};
-use twilight_model::{guild::Guild, id::GuildId};
+use crate::{
+    client::Client,
+    error::Error,
+    request::{Request, TryIntoRequest},
+    response::ResponseFuture,
+    routing::Route,
+};
+use twilight_model::{
+    guild::Guild,
+    id::{marker::GuildMarker, Id},
+};
 
 struct GetGuildFields {
     with_counts: bool,
@@ -9,12 +18,12 @@ struct GetGuildFields {
 #[must_use = "requests must be configured and executed"]
 pub struct GetGuild<'a> {
     fields: GetGuildFields,
-    guild_id: GuildId,
+    guild_id: Id<GuildMarker>,
     http: &'a Client,
 }
 
 impl<'a> GetGuild<'a> {
-    pub(crate) const fn new(http: &'a Client, guild_id: GuildId) -> Self {
+    pub(crate) const fn new(http: &'a Client, guild_id: Id<GuildMarker>) -> Self {
         Self {
             fields: GetGuildFields { with_counts: false },
             guild_id,
@@ -34,11 +43,20 @@ impl<'a> GetGuild<'a> {
     ///
     /// [`Response`]: crate::response::Response
     pub fn exec(self) -> ResponseFuture<Guild> {
-        let request = Request::from_route(&Route::GetGuild {
-            guild_id: self.guild_id.get(),
-            with_counts: self.fields.with_counts,
-        });
+        let http = self.http;
 
-        self.http.request(request)
+        match self.try_into_request() {
+            Ok(request) => http.request(request),
+            Err(source) => ResponseFuture::error(source),
+        }
+    }
+}
+
+impl TryIntoRequest for GetGuild<'_> {
+    fn try_into_request(self) -> Result<Request, Error> {
+        Ok(Request::from_route(&Route::GetGuild {
+            guild_id: self.guild_id.get(),
+            with_counts: self.fields.with_counts,
+        }))
     }
 }
